@@ -35,7 +35,6 @@
 #include <grl-log.h>
 
 #include "grl-net-mock-private.h"
-#include "grl-net-private.h"
 
 static GKeyFile *config = NULL;
 static GRegex *ignored_parameters = NULL;
@@ -62,12 +61,18 @@ get_url_mocked (GrlNetWc *self,
 
   if (ignored_parameters) {
     SoupURI *uri = soup_uri_new (url);
-    char *new_query = g_regex_replace (ignored_parameters,
-                                       soup_uri_get_query (uri), -1, 0,
-                                       "", 0, NULL);
-    soup_uri_set_query (uri, *new_query ? new_query : NULL);
-    new_url = soup_uri_to_string (uri, FALSE);
-    soup_uri_free (uri);
+    const char *query = soup_uri_get_query (uri);
+    if (query) {
+      char *new_query = g_regex_replace (ignored_parameters,
+                                         query, -1, 0,
+                                         "", 0, NULL);
+      soup_uri_set_query (uri, *new_query ? new_query : NULL);
+      new_url = soup_uri_to_string (uri, FALSE);
+      soup_uri_free (uri);
+      g_free (new_query);
+    } else {
+      new_url = g_strdup (url);
+    }
   } else {
     new_url = g_strdup (url);
   }
@@ -78,7 +83,9 @@ get_url_mocked (GrlNetWc *self,
                                      GRL_NET_WC_ERROR_NETWORK_ERROR,
                                      "%s",
                                      _("No mock definition found"));
+    g_free (new_url);
     g_simple_async_result_complete_in_idle (G_SIMPLE_ASYNC_RESULT (result));
+    g_object_unref (result);
     return;
   }
 
@@ -90,7 +97,9 @@ get_url_mocked (GrlNetWc *self,
                                      _("Could not find mock content %s"),
                                      error->message);
     g_error_free (error);
+    g_free (new_url);
     g_simple_async_result_complete_in_idle (G_SIMPLE_ASYNC_RESULT (result));
+    g_object_unref (result);
     return;
   }
   if (data_file[0] != '/') {
@@ -107,6 +116,8 @@ get_url_mocked (GrlNetWc *self,
                                      "%s",
                                      _("Could not access mock content"));
     g_simple_async_result_complete_in_idle (G_SIMPLE_ASYNC_RESULT (result));
+    g_object_unref (result);
+    g_free (new_url);
     if (data_file)
       g_free (data_file);
     if (full_path)
@@ -122,6 +133,7 @@ get_url_mocked (GrlNetWc *self,
                                              new_url,
                                              NULL);
   g_simple_async_result_complete_in_idle (G_SIMPLE_ASYNC_RESULT (result));
+  g_object_unref (result);
 }
 
 void
@@ -224,6 +236,9 @@ void init_mock_requester (GrlNetWc *self)
                    "for ignored query parameters: %s", error->message);
       g_clear_error (&error);
     }
+
+    g_strfreev (parameter_names);
+    g_string_free (pattern, TRUE);
   }
 
   /* Find base path for mock data. */
