@@ -256,7 +256,7 @@ grl_metadata_key_setup_system_keys (GrlRegistry *registry)
   grl_registry_register_metadata_key_full (registry,
                                            g_param_spec_int ("duration",
                                                              "Duration",
-                                                             "Media duration",
+                                                             "Media duration in seconds",
                                                              0, G_MAXINT,
                                                              0,
                                                              G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE),
@@ -266,7 +266,7 @@ grl_metadata_key_setup_system_keys (GrlRegistry *registry)
   grl_registry_register_metadata_key_full (registry,
                                            g_param_spec_int ("height",
                                                              "Height",
-                                                             "Height of media ('y' resolution)",
+                                                             "Height of media ('y' axis)",
                                                              0, G_MAXINT,
                                                              0,
                                                              G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE),
@@ -296,7 +296,7 @@ grl_metadata_key_setup_system_keys (GrlRegistry *registry)
   grl_registry_register_metadata_key_full (registry,
                                            g_param_spec_int ("width",
                                                              "Width",
-                                                             "Width of media ('x' resolution)",
+                                                             "Width of media ('x' axis)",
                                                              0, G_MAXINT,
                                                              0,
                                                              G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE),
@@ -422,7 +422,7 @@ grl_metadata_key_setup_system_keys (GrlRegistry *registry)
   grl_registry_register_metadata_key_full (registry,
                                            g_param_spec_int ("track-number",
                                                              "Track number",
-                                                             "Track number  inside the album",
+                                                             "Track number inside the album",
                                                              1, G_MAXINT,
                                                              1,
                                                              G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE),
@@ -448,6 +448,69 @@ grl_metadata_key_setup_system_keys (GrlRegistry *registry)
                                            GRL_METADATA_KEY_START_TIME,
                                            NULL);
 
+  grl_registry_register_metadata_key_full (registry,
+                                           g_param_spec_boolean ("favourite",
+                                                                 "Favourite",
+                                                                 "Whether or not the element was marked as favourite",
+                                                                 FALSE,
+                                                                 G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE),
+                                           GRL_METADATA_KEY_FAVOURITE,
+                                           NULL);
+
+  grl_registry_register_metadata_key_full (registry,
+                                           g_param_spec_string ("region",
+                                                                "Region",
+                                                                "Region in which the media was published",
+                                                                NULL,
+                                                                G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE),
+                                           GRL_METADATA_KEY_REGION,
+                                           NULL);
+
+  grl_registry_register_metadata_key_full (registry,
+                                           g_param_spec_string ("keyword",
+                                                                "keyword",
+                                                                "A keyword describing the media",
+                                                                NULL,
+                                                                G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE),
+                                           GRL_METADATA_KEY_KEYWORD,
+                                           NULL);
+
+  grl_registry_register_metadata_key_full (registry,
+                                           g_param_spec_string ("performer",
+                                                                "performer",
+                                                                "An actor performing in the movie",
+                                                                NULL,
+                                                                G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE),
+                                           GRL_METADATA_KEY_PERFORMER,
+                                           NULL);
+
+  grl_registry_register_metadata_key_full (registry,
+                                           g_param_spec_string ("producer",
+                                                                "Producer",
+                                                                "Producer of the movie",
+                                                                NULL,
+                                                                G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE),
+                                           GRL_METADATA_KEY_PRODUCER,
+                                           NULL);
+
+  grl_registry_register_metadata_key_full (registry,
+                                           g_param_spec_string ("director",
+                                                                "Director",
+                                                                "Director of the movie",
+                                                                NULL,
+                                                                G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE),
+                                           GRL_METADATA_KEY_DIRECTOR,
+                                           NULL);
+
+  grl_registry_register_metadata_key_full (registry,
+                                           g_param_spec_string ("original-title",
+                                                                "Original Title",
+                                                                "Original, untranslated title of the movie",
+                                                                NULL,
+                                                                G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE),
+                                           GRL_METADATA_KEY_ORIGINAL_TITLE,
+                                           NULL);
+
   /* Create the relations */
   grl_registry_register_metadata_key_relation (registry,
                                                GRL_METADATA_KEY_URL,
@@ -467,7 +530,12 @@ grl_metadata_key_setup_system_keys (GrlRegistry *registry)
   grl_registry_register_metadata_key_relation (registry,
                                                GRL_METADATA_KEY_URL,
                                                GRL_METADATA_KEY_START_TIME);
-
+  grl_registry_register_metadata_key_relation (registry,
+                                               GRL_METADATA_KEY_REGION,
+                                               GRL_METADATA_KEY_PUBLICATION_DATE);
+  grl_registry_register_metadata_key_relation (registry,
+                                               GRL_METADATA_KEY_REGION,
+                                               GRL_METADATA_KEY_CERTIFICATE);
 }
 
 /**
