@@ -366,10 +366,7 @@ multiple_search_cb (GrlSource *source,
 
   if (msd->cancelled) {
     GRL_DEBUG ("operation is cancelled or already finished, skipping result!");
-    if (media) {
-      g_object_unref (media);
-      media = NULL;
-    }
+    g_clear_object (&media);
     if (operation_done) {
       /* This was the last result and the operation is cancelled
 	 so we don't have anything else to do*/
@@ -499,7 +496,7 @@ media_from_uri_cb (GrlSource *source,
 
 /**
  * grl_multiple_search:
- * @sources: (element-type Grl.Source) (allow-none):
+ * @sources: (element-type GrlSource) (allow-none):
  * a #GList of #GrlSource<!-- -->s to search from (%NULL for all
  * searchable sources)
  * @text: the text to search for
@@ -602,7 +599,7 @@ multiple_search_cancel_cb (struct MultipleSearchData *msd)
 
 /**
  * grl_multiple_search_sync:
- * @sources: (element-type Grl.Source) (allow-none):
+ * @sources: (element-type GrlSource) (allow-none):
  * a #GList of #GrlSource<!-- -->s where to search from (%NULL for all
  * available sources with search capability)
  * @text: the text to search for
@@ -615,7 +612,7 @@ multiple_search_cancel_cb (struct MultipleSearchData *msd)
  *
  * This method is synchronous.
  *
- * Returns: (element-type Grl.Media) (transfer full): a list with #GrlMedia elements
+ * Returns: (element-type GrlMedia) (transfer full): a list with #GrlMedia elements
  *
  * Since: 0.2.0
  */
